#ifndef PROFILERINFORMATION_H
#define PROFILERINFORMATION_H

#include <utility>
#include <list>
#include <vector>
#include <map>
#include <set>
#include <string>

#include <cstddef>

#include <pro.h>
#include <ida.hpp>
#include <ua.hpp>


class InstructionInformation
{
	public:
		InstructionInformation() : numeric_count(0), pointer_count(0), other_count(0) {};
		virtual ~InstructionInformation() {};

		inline int isNumeric() { return numeric_count>0 && pointer_count==0 && other_count==0; }
		inline void setNumericCount(int nc) { numeric_count=nc;}
		inline void setPointerCount(int pc) { pointer_count=pc;}
		inline void setOtherCount(int oc) { other_count=oc;}
		inline long long getNumericCount() { return numeric_count; }
		inline long long getPointerCount() { return pointer_count; }
		inline long long getOtherCount() { return other_count; }

	private:

		long long numeric_count;
		long long pointer_count;
		long long other_count;
};

class ProfilerInformation
{
	public:
		ProfilerInformation(const char*);
		virtual ~ProfilerInformation();

		inline InstructionInformation* GetInfo(ea_t addr) { return the_map[addr]; }

	protected:
		// add type information from profiler about loads
		void addProfileInformation(ea_t addr, long long nc, long long pc, long long oc);
		
		// add pre-dec info from profiler
		void profileAddressingInformation ( ea_t addr, int size, char *type, 
			char *scope, ea_t the_const, char* field, ea_t real_const);

		inline void SetInfo(ea_t addr, InstructionInformation* ii) { the_map[addr]=ii; }

	private:

		// annotations that are used for numeric type propogation.
		std::map<ea_t,InstructionInformation*> 	the_map;	
	
		// annotations to spew back out verbatim.
		std::set<std::string>			constant_info;	

		std::string filename;

};

#endif

